import mdtraj as md
from tqdm import tqdm
import click
import numpy as np
import pickle
from sklearn.cluster import OPTICS



@click.command()
@click.argument('traj')
@click.argument('top')
@click.argument('output')
@click.option('--output_top', default=None)
@click.option('--stride', default=10)
@click.option('--cluster_number', default=5)
def cluster(traj, top, output, stride, cluster_number, output_top):
    subsampled = md.load(traj, top=top, stride=stride)
    selection = subsampled.topology.select('protein and name CA')

    print(subsampled)

    distances = np.empty((subsampled.n_frames, subsampled.n_frames))
    for i in tqdm(range(subsampled.n_frames)):
        distances[i] = md.rmsd(subsampled, subsampled, i, atom_indices=selection)

    #subsampled.superpose(subsampled)
    # Now that distances are computed, put molecules in good orientation
    subsampled.image_molecules()
        
    clust = OPTICS(min_samples=100, metric='precomputed')
    clust.fit(distances)
    # iterate over cluster
    # get lowest reachabillities
    # and cluster size
    centers = []
    for c in clust.cluster_hierarchy_:
        start,end = c
        i = np.argmin(clust.reachability_[clust.ordering_][start:end + 1])
        centers.append((i, end - start))

    # sort from largest to smallest
    centers = sorted(centers, key = lambda c: -c[1])
    
    # get topology
    if output_top is None:
        top = subsampled.topology
    else:
        top = md.load(output_top).topology


    protein = subsampled.topology.select('protein')
    print(len(protein))
    leaders = md.Trajectory(
        xyz=np.empty((0, len(protein), 3)), 
        topology=top,
        unitcell_lengths=np.array([]).reshape((0,3)),
        unitcell_angles=np.array([]).reshape((0,3))
    )
    
    for i,c in tqdm(enumerate(centers)):
        f = subsampled[c[0]].restrict_atoms(protein)
        leaders = leaders.join(f, check_topology=False) 
        print(f'Cluster {i} N = {c[1]}')


    leaders.save(f'{output}.pdb')

    with open('cluster_info.pb', 'wb') as f:
        pickle.dump(clust, f)

if __name__ == '__main__':
    cluster()

