import numpy as np
from peptidesim import PeptideSim, utilities
import mdtraj as md
import sys
import os
import sys
import gromacs
from shutil import copyfile, move
import fire
import textwrap


def run(name, seq, peptide_copies, peptide_density, debug=False, temperature=300):
    ps = PeptideSim(name, seq, peptide_copies, job_name='{}'.format(name))
    ps.mdrun_driver = 'gmx_mpi'
    ps.mpi_np = 2
    ps.mpiexec = 'mpiexec'
    ps.run_kwargs = {'ntomp': 4}
    ps.forcefield = 'charmm27'
    ps.water = 'tip3p'
    ps.peptide_density = peptide_density  # mg/ml
    ps.ion_concentration = 0.01  # PBS
    # add terminal amidation, acetylation
    chain_count = sum(peptide_copies)
    ps.initialize(amidate=True, acetylate=True)

    def ns_ts(ns, scale=None):
        '''nano seconds to timesteps. Use scale to turn down times for debugging'''
        if scale is None and debug:
            scale = 0.001
        else:
            scale = 1.0
        return str(max(1000, int(ns * 10**3 * 10**3 / 2 * scale)))


    ps.mpi_np = None


    plumed_file = ''
    traj = md.load_pdb(os.path.join(ps.dir_name, 'prep', 'dry_mixed.pdb'))
    for i in range(chain_count):
        atoms = traj.topology.select(f'protein and (backbone and chainid {i})')
        plumed_file += 'c{}: GROUP ATOMS={}\n'.format(i+1, ','.join([str(a+1) for a in atoms]))

    
    plumed_file += textwrap.dedent('''
    c1com: COM ATOMS=c1
    c2com: COM ATOMS=c2
    c3com: COM ATOMS=c3
    c4com: COM ATOMS=c4

    dist: DISTANCES GROUP=c1com,c2com,c3com,c4com LESS_THAN={GAUSSIAN R_0=0.01 D_0=1.25} MEAN LOWEST

    PRINT FILE=COLVAR STRIDE=2000 ARG=dist.lessthan,dist.mean,dist.lowest
    com2: COMMITTOR BASIN_LL1=2.95 BASIN_UL1=4.0 ARG=dist.lessthan


    ''')

    with open('plumed.dat', 'w') as f:
        f.write(plumed_file)
    
    ps.add_file('plumed.dat')
    
    ps.run(mdpfile='peptidesim_nvt.mdp',
           tag='nvt-committor',
           run_kwargs={'plumed': 'plumed.dat'},
           mdp_kwargs={'nsteps': ns_ts(1000), 'ref_t': temperature, 'nstfout': 0, 'nstxout': 2000})

if __name__ == '__main__':
    fire.Fire(run)
